function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise, MessageService) {
    var self = this;
    var loginUserId, loginUserName, loginIsSupOper, loginSupOperId;
    window.tekLoginProjectId = angular.copy(window.projectId);

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var updateLanguage = function () {
        var language = localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);
    };

    var redirectToRedirectionProduct = function (redirectUrl, defer) {
        if (window.ZhNativeInterface) {
            window.ZhNativeInterface.redirect(redirectUrl);
        } else {
            window.location = redirectUrl;
        }
        defer.resolve({});
    };

    var updateToken = function () {
        var authToken = Util.getUrlParameter('authToken');
        if (authToken) {
            localStorage.setItem('LOGIN_TOKEN', authToken);
        }
        return authToken;
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        updateLanguage();

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER');
            var redirectUrl = localStorage.getItem('LOGIN_REDIRECT_URL');
            var authToken = updateToken();

            if(redirectUrl && !redirectUrl.includes(window.location.hostname)) {
                redirectToRedirectionProduct(redirectUrl, defer);
            } else if(!user && !authToken) {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };
                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                ScreenService.showLoader();
                LibLoginService.startSession(user).then(function (response) {
                    ScreenService.hideLoader();
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    ScreenService.hideLoader();
                    //Expired or Invalid Token
                    if(error.data && (error.data.errorCode == 101 || error.data.errorCode == 100)) {
                        self.logout();
                    } else if(error.data.errorCode == 20){
                        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
                        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
                            type: 'INFORMATION',
                            actions: [ {
                                label: ScreenService.i18n("Ok"),
                                code: function(){
                                    self.logout()
                                }
                            } ]
                        });
                    } else{
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                if(response.LOGIN_USER && response.LOGIN_TOKEN) {
                    localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                    localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);
                }
                if(response.IS_SUPPORT_OPERATOR) {
                    loginIsSupOper = response.IS_SUPPORT_OPERATOR;
                    loginSupOperId = response.SUPPORT_OPERATOR_ID;
                }

                loginUserId   = response.USER_ID;
                loginUserName = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    MessageService.showMessageByCode('lib#ALE_MIGRATION_IS_PENDENT', null, { type: 'INFORMATION' });
                }

                defer.resolve(response);
            }
        }
    };

    this.logout = function() {
        ScreenService.showLoader();
        LibLoginService.logout().then(function () {
            ScreenService.hideLoader();
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_ERASE_HASH');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            LibUtilities.redirectToLogin();
        }).catch(function(error) {
            ScreenService.hideLoader();
            LibUtilities.showErrorMessage(error);
        });
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION', null, {
                type: 'INFORMATION',
                actions: [ {
                    label: ScreenService.i18n("Ok"),
                    code: self.logout
                } ]
            });
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    eventAggregator.subscribe("LibInvalidTokenSupportOperator", function (errorCode) {
        //Expired Token Support Operator
        MessageService.showCustomDialogByCode('lib#ALE_EXPIRED_SESSION_SUPPORT_OPERATOR', null, {
            type: 'INFORMATION',
            actions: [{
                label: ScreenService.i18n("Ok"),
                code: self.logout
            }]
        });
    });

    eventAggregator.subscribe("LibConcurrentAccess", function () {
        localStorage.setItem('LOGIN_ERASE_HASH', 'Y');
        MessageService.showCustomDialogByCode('lib#ALE_SESSION_WILL_BE_ENDED', null, {
            type: 'INFORMATION',
            actions: [ {
                label: ScreenService.i18n("Ok"),
                code: function(){
                    self.logout()
                }
            } ]
        });
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var message = afterLogin ? 'ERR_EXPIRED_SESSION_LOGIN' : 'ERR_ATTEMPT_END';
        MessageService.getMessageByCode('lib#' + message).then(function(loadingMessage) {
            ScreenService.changeLoadingMessage(ScreenService.i18n(loadingMessage));
            ScreenService.showLoader();
            Util.doAsync(self.logout, 5000);
        });
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            var userId   = loginSupOperId || loginUserId;
            ScreenService.showLoader();
            LibLoginService.validateUser(userId, password, loginIsSupOper)
                .then(function() {
                    // Requisição getModulesVersion chamada para atualizar o tempo de sessão local (produto ou módulo)
                    LibLoginService.getModulesVersion();
                    ScreenService.hideLoader();
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function(error) {
                    ScreenService.hideLoader();
                    if(error.data && error.data.errorCode && error.data.errorCode == 20) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibConcurrentAccess');
                    } else if(error.data && error.data.errorCode && error.data.errorCode == 102) {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        eventAggregator.publish('LibInvalidTokenSupportOperator');
                    } else {
                        var params = {attempts: ++expiredSessionWidget.attempts};
                        MessageService.showMessageByCode('lib#ERR_ATTEMPT_PARTIAL', params, {type: 'ERROR'});
                        if (expiredSessionWidget.attempts >= 3) {
                            logoutExpiredSession();
                        }
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    };

    /**
     * @param user -> Object or null.
     *                {
     *                    NAME:  'User Name',
     *                    EMAIL: 'user.email@teknisa.com'
     *                }
     * @param accessProfile -> Array of objects or null. ID is the column NRPERFILACESSO from table PERFILACESSO.
     *                         [
     *                             {
     *                                 ID: '1',
     *                                 IDPERFILACESSOADM: 'S'
     *                             },
     *                             {
     *                                 ID: '2',
     *                                 IDPERFILACESSOADM: 'N'
     *                             }
     *                         ]
     * @param otherSessionValues -> Object or null. PS.: CDFILIALSELECIONADA not exists in session. Use CDFILIAL.
     *                              {
     *                                  CDFILIAL:  '9999',
     *                                  NRPARCNEGOCIO: '1',
     *                                  CDPICTPROD: 'a.a.aa.aaa'
     *                              }
     * @returns Promise
     */
    this.updateSession = function(user, accessProfile, otherSessionValues) {
        var defer = ZHPromise.defer();
        ScreenService.showLoader();
        LibLoginService.updateSession(user, accessProfile, otherSessionValues).then(function(response) {
            ScreenService.hideLoader();
            if(response && response.dataset && response.dataset.sessionUpdated) {
                var sessionUpdated = response.dataset.sessionUpdated[0];
                localStorage.setItem('LOGIN_USER', sessionUpdated);
                defer.resolve();
            } else {
                defer.reject(response);
            }
        }).catch(function(error) {
            ScreenService.hideLoader();
            defer.reject(error);
        });

        return defer.promise;
    };

    this.expiredMenuOnClick = function() {
        MessageService.showMessageByCode('lib#ALE_EXPIRED_MENU', null, { type: 'ERROR' } );
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});